package net.udby.datetime.ejb;

import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TemporalType;
import javax.persistence.TypedQuery;

import net.udby.datetime.DateTimeProvider;
import net.udby.datetime.jpa.Item;

@Stateless
public class ItemServiceEjb implements ItemService {
	private static final long T_GREEN = 5000L;
	private static final long T_YELLOW = 30000L;

	@PersistenceContext
	private EntityManager em;
	
	private DateTimeProvider dateTimeProvider = DateTimeProvider.DEFAULT;
	
	public ItemServiceEjb() {
	}

	@Override
	public List<Item> findActiveItems() {
		TypedQuery<Item> q = em.createNamedQuery(Item.QRY_FIND_ACTIVE, Item.class);
		q.setParameter("now", dateTimeProvider.currentTimeCalendar(), TemporalType.TIMESTAMP);
	 
		return q.getResultList();
	}

	@Override
	public Item create(Item item) {
		item.setCreated(dateTimeProvider.currentTimeCalendar());
		em.persist(item);
		em.flush();
		return item;
	}

	@Override
	public Status getStatus(Item item) {
		final long tdiff = dateTimeProvider.currentTimeMilliseconds() - item.getCreated().getTimeInMillis();
		if (tdiff < 0) {
			throw new IllegalStateException("Negative Item age: " + tdiff + ", item.id=" + item.getId());
		} else if (tdiff < T_GREEN) {
			return Status.GREEN;
		} else if (tdiff < T_YELLOW) {
			return Status.YELLOW;
		} else {
			return Status.RED;
		}
	}
	
	void setEm(EntityManager em) {
		this.em = em;
	}

	void setDateTimeProvider(DateTimeProvider dateTimeProvider) {
		this.dateTimeProvider = dateTimeProvider;
	}
}
